/**
 * Copyright (c) 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.admin.junit;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import org.junit.FixMethodOrder;
import org.junit.Rule;
import org.junit.Test;
import org.junit.runners.MethodSorters;
import org.ow2.petals.admin.api.artifact.ArtifactState.State;
import org.ow2.petals.admin.api.artifact.Component;
import org.ow2.petals.admin.api.artifact.Component.ComponentType;
import org.ow2.petals.admin.api.artifact.ServiceAssembly;
import org.ow2.petals.admin.api.artifact.SharedLibrary;

/**
 * Unit tests of {@link PetalsAdministrationApi}
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
// MethodSorter 'NAME_ASCENDING' required to check that the artifact registry is correctly cleared at the end of a unit
// test.
@FixMethodOrder(MethodSorters.NAME_ASCENDING)
public class PetalsAdministrationApiTest {

    /**
     * A Petals Administration Mock dedicated to the test 'artifactRegistryClearedAtTheEnd' to check that the artifact
     * registry is correctly cleared when leaving the unit test.
     */
    @Rule
    public final PetalsAdministrationApi globalAdminApi = new PetalsAdministrationApi();

    @Rule
    public final PetalsAdministrationApi adminApiOne = new PetalsAdministrationApi();

    @Rule
    public final PetalsAdministrationApi adminApiTwo = new PetalsAdministrationApi();

    /**
     * <p>
     * Check that the artifact registry is correctly cleared when leaving the unit test.
     * </p>
     * <p>
     * In the test {@link #artifactRegistryClearedAtTheEnd_0()}, an artifact is added to the registry, the check will be
     * done in {@link #artifactRegistryClearedAtTheEnd_1()}.
     * </p>
     */
    @Test
    public void artifactRegistryClearedAtTheEnd_0() {

        final Component component = new Component("component-name", ComponentType.BC);
        component.setState(State.STARTED);
        this.globalAdminApi.registerArtifact(component);

        // The check is done in artifactRegistryClearedAtTheEnd_1
    }

    /**
     * See {@link #artifactRegistryClearedAtTheEnd_0()}
     */
    @Test
    public void artifactRegistryClearedAtTheEnd_1() {

        assertEquals("Unexpected number of artifact in the artifact registry", 0, this.globalAdminApi
                .getArtifactRegistry().list().length);

        // The registration a the same component should succeeds
        final Component componentOne = new Component("component-name", ComponentType.BC);
        componentOne.setState(State.STARTED);
        this.globalAdminApi.registerArtifact(componentOne);

    }

    /**
     * Check that the artifact registries of two Petals Administration Mock are correctly isolated.
     */
    @Test
    public void oneArtifactRegistryPerPetalsAdminMockInstance() {

        final Component componentOne = new Component("component-name", ComponentType.BC);
        componentOne.setState(State.STARTED);
        this.adminApiOne.registerArtifact(componentOne);

        assertEquals(1, this.adminApiOne.getArtifactRegistry().list().length);
        assertEquals(0, this.adminApiTwo.getArtifactRegistry().list().length);

        this.adminApiTwo.registerArtifact(new SharedLibrary("shared-library", "version"));

        assertEquals(1, this.adminApiOne.getArtifactRegistry().list().length);
        assertEquals(1, this.adminApiTwo.getArtifactRegistry().list().length);
    }

    /**
     * <p>
     * Unregister an artifact previously registered.
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error occurs,</li>
     * <li>the artifact is correctly registered</li>
     * </ul>
     * </p>
     */
    @Test
    public void unregisterAnArtifactPreviouslyRegistered() {

        final Component componentOne = new Component("component-name", ComponentType.BC);
        this.adminApiOne.registerArtifact(componentOne);

        assertEquals(1, this.adminApiOne.getArtifactRegistry().list().length);

        assertTrue(this.adminApiOne.unregisterArtifact(componentOne));
        assertEquals(0, this.adminApiOne.getArtifactRegistry().list().length);

    }

    /**
     * <p>
     * Unregister an inexisting component.
     * </p>
     * <p>
     * Expected results: <code>false</code> is returned because the component is not registered.
     * </p>
     */
    @Test
    public void unregisterAnUnexistingArtifact() {

        assertFalse(this.adminApiOne.unregisterArtifact(new Component("component-name", ComponentType.BC)));

    }

    /**
     * <p>
     * Unregister an artifact with the same name than an artifact previously registered, but with a different type.
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error occurs,</li>
     * <li>the artifact is not unregistered.</li>
     * </ul>
     * </p>
     */
    @Test
    public void unregisterSamenameDifferentType() {

        final String artifactName = "artifact-name";
        final Component componentOne = new Component(artifactName, ComponentType.BC);
        this.adminApiOne.registerArtifact(componentOne);

        assertEquals(1, this.adminApiOne.getArtifactRegistry().list().length);

        assertFalse(this.adminApiOne.unregisterArtifact(new ServiceAssembly(artifactName)));
        assertEquals(1, this.adminApiOne.getArtifactRegistry().list().length);

        assertFalse(this.adminApiOne.unregisterArtifact(new Component(artifactName, ComponentType.SE)));
        assertEquals(1, this.adminApiOne.getArtifactRegistry().list().length);

    }

}
